ChangeLog
=========

All notable changes to this project will be documented in this file.
This project adheres to [Semantic Versioning](http://semver.org/).

Conventions when editing this file.
-----------------------------------

Please follow the listed conventions when editing this file:

- one section per version
- reverse chronological order (latest entry on top)
- write all dates in iso notation (`YYYY-MM-DD`)
- each version should group changes according to their impact:
    - `Added` for new features.
    - `Changed` for changes in existing functionality.
    - `Deprecated` for once-stable features removed in upcoming releases.
    - `Removed` for deprecated features removed in this release.
    - `Fixed` for any bug fixes.
    - `Security` to invite users to upgrade in case of vulnerabilities.

v0.12.0 (2024-04-18)
====================

Added
-----

- organisation administration (edit and delete)
- sync user account data to ticket database also upon account validation
- strike-through rendering to `MarkdownRenderer`
- image attributes extension for `MarkdownRenderer`
- error handling middleware for better error logging

Changed
-------

- better error message if user or organisation cannot be displayed
- Refactoring: merge tickets module into hub module
- base uri for organisation operations
- scalafix configuration update
- documentation for CSRF protection on failure handler to reflect correct
  behaviour
- documentation wording

Removed
-------

- now obsolete workaround for twirl and scala-xml from build file
- unused packaging settings from build file

Fixed
-----

- broken source tarball downloads
- set correct password reset link in email
- discarded non unit value in `DarcsCommandsTest`
- fatal warning in the `MarkdownRenderer`
- missing logback on html-utils

v0.11.0 (2024-02-02)
====================

This release contains BREAKING CHANGES in the database structure!

To mitigate the effect create a backup of your database first. Then follow
these instructions:

1. stop the service
2. dump the data of your database only (`pg_dump -a dbname > data.sql`)
3. !!! Remove the `flyway_schema_history` tables from the dump !!!
4. drop the database and create a new one with the same name
5. apply the update
6. start the service
7. stop the service after the migrations have been applied
8. restore the data (`cat data.sql | psql dbname`)

Please note that you might need additional parameters for `pg_dump` and `psql`
depending on your database configuration like username, password or prepended
`sudo` command.

Changed
-------

- **BREAKING CHANGE** remove unnecessary quoting in sql

v0.10.0 (2024-02-01)
====================

Added
-----

- experimental support for ed25519 ssh keys
- full name field to account
- permission model to enable modelling privileges
- base for organisations that can own repositories

Changed
-------

- release guide (i.e. changelog update process)

Removed
-------

- creation of empty `production.conf` file upon package install

Fixed
-----

- deprecation warning about ResourceFixture

v0.9.0 (2023-11-13)
===================

Added
-----

- example script for systemd service
- RSS feed for repositories
- milestones can be closed and re-opened
- show tickets on milestone detail page
- increasing time delay for failing login attempts
- unlocking of locked accounts
- more detailed error page for users upon possible CSRF errors
- forgot password functionality

Changed
-------

- major code reformat due to change of indentation for Scala to 4 spaces
- updates to code style guidelines
- show full path in title attribute of web page in repository file browser

Fixed
-----

- HTML/CSS issues
- RSS whitespace issues
- extension for filename when downloading repository archive not correct
- unicode filename issues in repository archive
- unicode issues in the repository file browser
- properly render html meta tags

v0.8.0 (2023-06-29)
===================

Added
-----

- basic administration commands via cli for the hub module
- basic sftp support for darcs operations via ssh
- more information regarding deployment in the README
- include build information (name, version, scala version) in hub and ticket
  modules
- example configuration file for running hub and ticket module on localhost

Changed
-------

- removal of integration test build configuration axis from sbt
- lots of dependency updates
- suppress diagnostic logging output from logback
- default configuration now contains defaults for all settings

v0.7.0 (2023-05-28)
===================

Added
-----

- support for labels, milestones and tickets :-)
- compiler flags to enforce syntax rules and prevent mixing of syntax styles
- link to related patches if a `Fixes` comment is found in the patch view

Changed
-------

- switch to Contributor Covenant Code of Conduct
- adjust and extend contribution guide
- lots of updates and refactoring some modules

v0.6.0 (2023-03-26)
===================

Added
-----

- localisation support for signed in users
- more information on branches (forks) on repository pages

v0.5.0 (2023-03-23)
===================

This release contains BREAKING CHANGES in the database structure!

Added
-----

- extensions to the markdown renderer for:
   - anchors for headings
   - table rendering
   - task list items
- linkable line number for file viewer

Changed
-------

- **BREAKING CHANGE** configuration file format
- **BREAKING CHANGE** move database tables to custom schemas
- move database migration files to custom path
- adjust migration tooling accordingly
- move some types and classes to other modules
- limit length of patch comment shown on repository overview page

v0.4.0 (2023-01-09)
===================

Added
-----

- show parent repository from which the current one was forked on the
  repository overview page

Changed
-------

- store the CSRF key on disk and load it if present
- lots of dependency updates

Removed
-------

- randomised id attribute of CSRF form fields
- sbt-site and related plugins due to scala-xml upgrade blocking

v0.3.0 (2022-11-16)
===================

Added
-----

- use the CSRF protection middleware of http4s
- always render markdown files when browsing the repository
- extend history browsing to show patch details (diffs)

Fixed
-----

- history summary shows wrong number of lines that were added/removed
- relative links in rendered markdown README are broken on overview page

Security
--------

- Update Apache Mina SSHD to 2.9.2 (CVE-2022-45047)

v0.2.0 (2022-11-03)
===================

Changed
-------

- show ssh clone option only for repository owners
- don't use a hostname for the ssh server to bind to all addresses

Fixed
-----

- wrong key in logback configuration
- customised local developer application.conf in package
- ssh clone url has wrong hostname
- issues with html and css in the footer
- ssh banner for shell shows wrong clone syntax

v0.1.0 (2022-11-01)
===================

- initial release

