/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.darcs

import java.io.IOException
import java.nio.file.*
import java.nio.file.attribute.BasicFileAttributes

import cats.syntax.all.*

trait TestHelpers {

    /** Delete the given directory recursively.
      *
      * @param path
      *   The path on the filesystem to the directory that shall be deleted.
      * @return
      *   `true` if the directory was deleted.
      */
    protected def deleteDirectory(path: Path): Boolean =
        if (path.toString.trim =!= "/") {
            Files.walkFileTree(
                path,
                new FileVisitor[Path] {
                    override def visitFileFailed(file: Path, exc: IOException): FileVisitResult =
                        FileVisitResult.CONTINUE

                    override def visitFile(file: Path, attrs: BasicFileAttributes): FileVisitResult = {
                        Files.delete(file)
                        FileVisitResult.CONTINUE
                    }

                    override def preVisitDirectory(dir: Path, attrs: BasicFileAttributes): FileVisitResult =
                        FileVisitResult.CONTINUE

                    override def postVisitDirectory(dir: Path, exc: IOException): FileVisitResult = {
                        Files.delete(dir)
                        FileVisitResult.CONTINUE
                    }
                }
            )
            Files.deleteIfExists(path)
        } else false
}
