CREATE TABLE hub.repositories
(
  id          BIGINT                   GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
  name        CHARACTER VARYING(64)    NOT NULL,
  owner       UUID                     NOT NULL,
  is_private  BOOLEAN                  NOT NULL DEFAULT FALSE,
  description CHARACTER VARYING(254),
  vcs_type    CHARACTER VARYING(16)    NOT NULL,
  website     TEXT,
  created_at  TIMESTAMP WITH TIME ZONE NOT NULL,
  updated_at  TIMESTAMP WITH TIME ZONE NOT NULL,
  CONSTRAINT repositories_unique_owner_name UNIQUE (owner, name),
  CONSTRAINT repositories_fk_uid FOREIGN KEY (owner)
    REFERENCES hub.accounts (uid) ON UPDATE CASCADE ON DELETE CASCADE
)
WITH (
  OIDS=FALSE
);

COMMENT ON TABLE hub.repositories IS 'All repositories i.e. their metadata are stored within this table. The combination of owner id and repository name must always be unique.';
COMMENT ON COLUMN hub.repositories.id IS 'An auto generated primary key.';
COMMENT ON COLUMN hub.repositories.name IS 'A repository name must start with a letter or number and must contain only alphanumeric ASCII characters as well as minus or underscore signs. It must be between 2 and 64 characters long.';
COMMENT ON COLUMN hub.repositories.owner IS 'The unique ID of the user account that owns the repository.';
COMMENT ON COLUMN hub.repositories.is_private IS 'A flag indicating if this repository is private i.e. only visible / accessible for accounts with appropriate permissions.';
COMMENT ON COLUMN hub.repositories.description IS 'An optional short text description of the repository.';
COMMENT ON COLUMN hub.repositories.vcs_type IS 'The type of the underlying DVCS that manages the repository.';
COMMENT ON COLUMN hub.repositories.website IS 'An optional uri pointing to a website related to the repository / project.';
COMMENT ON COLUMN hub.repositories.created_at IS 'The timestamp of when the repository was created.';
COMMENT ON COLUMN hub.repositories.updated_at IS 'A timestamp when the repository was last changed.';
