CREATE TABLE hub.organisations
(
  id          UUID                     NOT NULL,
  name        CHARACTER VARYING(32)    NOT NULL,
  owner       UUID                     NOT NULL,
  full_name   CHARACTER VARYING(128),
  description CHARACTER VARYING(254),
  is_private  BOOLEAN                  NOT NULL DEFAULT FALSE,
  website     TEXT,
  created_at  TIMESTAMP WITH TIME ZONE NOT NULL,
  updated_at  TIMESTAMP WITH TIME ZONE NOT NULL,
  CONSTRAINT organisations_pk          PRIMARY KEY (id),
  CONSTRAINT organisations_unique_name UNIQUE (name),
  CONSTRAINT organisations_fk_uid FOREIGN KEY (owner)
    REFERENCES hub.accounts (uid) ON UPDATE CASCADE ON DELETE CASCADE
)
WITH (
  OIDS=FALSE
);

CREATE INDEX organisations_private ON hub.organisations (is_private);

COMMENT ON TABLE hub.organisations IS 'All organisations live within this table.';
COMMENT ON COLUMN hub.organisations.id IS 'A globally unique ID for the organisation.';
COMMENT ON COLUMN hub.organisations.name IS 'A name (like a username) between 2 and 32 characters which must be globally unique, contain only lowercase alphanumeric characters and start with a character.';
COMMENT ON COLUMN hub.organisations.owner IS 'The unique ID of the user account owning the organisation.';
COMMENT ON COLUMN hub.organisations.full_name IS 'An optional full name for the organisation.';
COMMENT ON COLUMN hub.organisations.description IS 'An optional description of the organisation.';
COMMENT ON COLUMN hub.organisations.is_private IS 'A flag indicating if this organisation is private i.e. only visible / accessible for accounts with appropriate permissions.';
COMMENT ON COLUMN hub.organisations.website IS 'An optional uri pointing to a website related to the organisation.';
COMMENT ON COLUMN hub.organisations.created_at IS 'The timestamp of when the organisation was created.';
COMMENT ON COLUMN hub.organisations.updated_at IS 'A timestamp when the organisation was last changed.';

CREATE TABLE hub.organisation_admins
(
  organisation UUID NOT NULL,
  admin        UUID NOT NULL,
  CONSTRAINT organisation_admins_pk     PRIMARY KEY (organisation, admin),
  CONSTRAINT organisation_admins_fk_oid FOREIGN KEY (organisation)
    REFERENCES hub.organisations (id) ON UPDATE CASCADE ON DELETE CASCADE,
  CONSTRAINT organisation_admins_fk_uid FOREIGN KEY (admin)
    REFERENCES hub.accounts (uid) ON UPDATE CASCADE ON DELETE CASCADE
)
WITH (
  OIDS=FALSE
);

COMMENT ON TABLE hub.organisation_admins IS 'A mapping table to allow multiple users administering an organisation.';
COMMENT ON COLUMN hub.organisation_admins.organisation IS 'The globally unique ID of the organisation.';
COMMENT ON COLUMN hub.organisation_admins.admin IS 'The unique ID of one of the user accounts allowed to administer the organisation.';
