/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.hub

import java.util.UUID

import cats.effect.*
import de.smederee.email.EmailAddress
import de.smederee.security.*
import doobie.*
import doobie.implicits.*
import doobie.postgres.implicits.*

final class DoobieSignupRepository[F[_]: Sync](tx: Transactor[F]) extends SignupRepository[F] {
    given Meta[EmailAddress] = Meta[String].timap(EmailAddress.apply)(_.toString)
    given Meta[PasswordHash] = Meta[String].timap(PasswordHash.apply)(_.toString)
    given Meta[UserId]       = Meta[UUID].timap(UserId.apply)(_.toUUID)
    given Meta[Username]     = Meta[String].timap(Username.apply)(_.toString)

    override def createAccount(account: Account, hash: PasswordHash): F[Int] =
        sql"""INSERT INTO hub.accounts (uid, name, email, password, created_at, updated_at, validated_email) VALUES(${account.uid}, ${account.name}, ${account.email}, $hash, NOW(), NOW(), ${account.validatedEmail})""".update.run
            .transact(tx)

    override def findEmail(address: EmailAddress): F[Option[EmailAddress]] =
        sql"""SELECT email FROM hub.accounts WHERE email = $address""".query[EmailAddress].option.transact(tx)

    override def findUsername(name: Username): F[Option[Username]] =
        sql"""SELECT name FROM hub.accounts WHERE name = $name""".query[Username].option.transact(tx)
}
