/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.hub

/** The human readable full name of an actor (account, organisation) which must be non empty and is limited to 128
  * characters.
  */
opaque type FullName = String
object FullName {
    val MaximumLength: Int = 128

    /** Create an instance of FullName from the given String type.
      *
      * @param source
      *   An instance of type String which will be returned as a FullName.
      * @return
      *   The appropriate instance of FullName.
      */
    def apply(source: String): FullName = source

    /** Try to create an instance of FullName from the given String.
      *
      * @param source
      *   A String that should fulfil the requirements to be converted into a FullName.
      * @return
      *   An option to the successfully converted FullName.
      */
    def from(source: String): Option[FullName] = Option(source).filter(_.nonEmpty).filter(_.length <= MaximumLength)
}
