/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.hub

import cats.data.*
import cats.syntax.all.*
import de.smederee.email.EmailAddress
import de.smederee.hub.forms.*
import de.smederee.hub.forms.types.*

/** Data container for the form used to reset a password.
  *
  * @param email
  *   The email address to which the reset link shall be sent that must exist and belong to an account.
  */
final case class ResetPasswordForm(email: EmailAddress)

object ResetPasswordForm extends FormValidator[ResetPasswordForm] {
    val fieldEmail: FormField = FormField("email")

    override def validate(data: Map[String, String]): ValidatedNec[FormErrors, ResetPasswordForm] = {
        val email: ValidatedNec[FormErrors, EmailAddress] = data
            .get(fieldEmail)
            .fold(FormFieldError("No email address given!").invalidNec)(s =>
                EmailAddress.from(s).fold(FormFieldError("Invalid email address!").invalidNec)(_.validNec)
            )
            .leftMap(es => NonEmptyChain.of(Map(fieldEmail -> es.toList)))
        email.map(ResetPasswordForm.apply)
    }
}
