/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.hub.config

/** A configuration path describes a path within a configuration file and is used to determine locations of certain
  * configurations within a combined configuration file.
  */
opaque type ConfigurationPath = String
object ConfigurationPath {

    given Conversion[ConfigurationPath, String] = _.toString

    /** Create an instance of ConfigurationPath from the given String type.
      *
      * @param source
      *   An instance of type String which will be returned as a ConfigurationPath.
      * @return
      *   The appropriate instance of ConfigurationPath.
      */
    def apply(source: String): ConfigurationPath = source

    /** Try to create an instance of ConfigurationPath from the given String.
      *
      * @param source
      *   A String that should fulfil the requirements to be converted into a ConfigurationPath.
      * @return
      *   An option to the successfully converted ConfigurationPath.
      */
    def from(source: String): Option[ConfigurationPath] = Option(source).map(_.trim).filter(_.nonEmpty)
}
