/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.tickets

import de.smederee.security.UserId

/** Definition of a programmatic API for the ticket service which can be used to initialise and synchronise data with
  * the hub service.
  *
  * @tparam F
  *   A higher kinded type which wraps the actual return values.
  */
abstract class TicketServiceApi[F[_]] {

    /** Create a user in the ticket service or update an existing one if an account with the unique id already exists.
      *
      * @param user
      *   The user account that shall be created.
      * @return
      *   The number of affected database rows.
      */
    def createOrUpdateUser(user: TicketsUser): F[Int]

    /** Delete the given user from the ticket service.
      *
      * @param uid
      *   The unique id of the user account.
      * @return
      *   The number of affected database rows.
      */
    def deleteUser(uid: UserId): F[Int]

}
