/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.hub.forms

import cats.data.*
import cats.syntax.all.*
import de.smederee.hub.forms.types.*

import munit.*

final class FormErrorsTest extends FunSuite {

    test("FormErrors.empty must create an empty instance") {
        val noErrors = FormErrors.empty
        assert(noErrors.isEmpty)
    }

    test("FormErrors.fromNec must collapse given instances into a single one") {
        val a = NonEmptyChain.of(
            FormErrors.empty + (FormField("a") -> List(FormFieldError("Error a1"))),
            FormErrors.empty + (FormField("b") -> List(FormFieldError("Error b1")))
        )
        val b = NonEmptyChain.of(FormErrors.empty)
        val c = NonEmptyChain.of(
            Map(
                FormField("a") -> List(FormFieldError("Error a2")),
                FormField("b") -> List(FormFieldError("Error b2"))
            ),
            Map(FormField("b") -> List(FormFieldError("Error b3")))
        )
        val expected = Map(
            FormField("a") -> List(FormFieldError("Error a1"), FormFieldError("Error a2")),
            FormField("b") -> List(
                FormFieldError("Error b1"),
                FormFieldError("Error b2"),
                FormFieldError("Error b3")
            )
        )
        val combined = FormErrors.fromNec(a |+| b |+| c)
        assertEquals(combined, expected)
    }

    test("FormErrors.fromNel must collapse given instances into a single one") {
        val a = NonEmptyList.of(
            FormErrors.empty + (FormField("a") -> List(FormFieldError("Error a1"))),
            FormErrors.empty + (FormField("b") -> List(FormFieldError("Error b1")))
        )
        val b = NonEmptyList.of(FormErrors.empty)
        val c = NonEmptyList.of(
            Map(
                FormField("a") -> List(FormFieldError("Error a2")),
                FormField("b") -> List(FormFieldError("Error b2"))
            ),
            Map(FormField("b") -> List(FormFieldError("Error b3")))
        )
        val expected = Map(
            FormField("a") -> List(FormFieldError("Error a1"), FormFieldError("Error a2")),
            FormField("b") -> List(
                FormFieldError("Error b1"),
                FormFieldError("Error b2"),
                FormFieldError("Error b3")
            )
        )
        val combined = FormErrors.fromNel(a |+| b |+| c)
        assertEquals(combined, expected)
    }
}
