/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.ssh

import cats.effect.*
import cats.syntax.all.*
import de.smederee.TestTags.*
import de.smederee.hub.BaseSpec
import de.smederee.hub.Generators.*
import de.smederee.security.*
import doobie.*

final class DoobieSshAuthenticationRepositoryTest extends BaseSpec {
    test("findVcsRepositoryOwner must return the correct owner".tag(NeedsDatabase)) {
        genValidAccount.sample match {
            case Some(account) =>
                val dbConfig = configuration.database
                val tx = Transactor.fromDriverManager[IO](
                    driver = dbConfig.driver,
                    url = dbConfig.url,
                    user = dbConfig.user,
                    password = dbConfig.pass,
                    logHandler = None
                )
                val repo = new DoobieSshAuthenticationRepository[IO](tx)
                val test = for {
                    _     <- createAccount(account, PasswordHash("I am not a password hash!"), None, None)
                    owner <- repo.findVcsRepositoryOwner(account.name)
                } yield owner
                test.assertEquals(account.toVcsRepositoryOwner.some)
            case _ => fail("Could not generate data samples!")
        }
    }
}
