/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.tickets.config

import cats.effect.*
import cats.syntax.all.*
import de.smederee.TestTags.*
import de.smederee.tickets.BaseSpec
import org.flywaydb.core.Flyway

final class DatabaseMigratorTest extends BaseSpec {
    override def beforeEach(context: BeforeEach): Unit = {
        val dbConfig = configuration.database
        val flyway: Flyway =
            DatabaseMigrator.configureFlyway(dbConfig.url, dbConfig.user, dbConfig.pass).cleanDisabled(false).load()
        val _ = flyway.migrate()
        val _ = flyway.clean()
    }

    override def afterEach(context: AfterEach): Unit = {
        val dbConfig = configuration.database
        val flyway: Flyway =
            DatabaseMigrator.configureFlyway(dbConfig.url, dbConfig.user, dbConfig.pass).cleanDisabled(false).load()
        val _ = flyway.migrate()
        val _ = flyway.clean()
    }

    test("DatabaseMigrator must update available outdated database".tag(NeedsDatabase)) {
        val dbConfig = configuration.database
        val migrator = new DatabaseMigrator[IO]
        val test     = migrator.migrate(dbConfig.url, dbConfig.user, dbConfig.pass)
        test.map(result => assert(result.migrationsExecuted > 0))
    }

    test("DatabaseMigrator must not update an up to date database".tag(NeedsDatabase)) {
        val dbConfig = configuration.database
        val migrator = new DatabaseMigrator[IO]
        val test = for {
            _ <- migrator.migrate(dbConfig.url, dbConfig.user, dbConfig.pass)
            r <- migrator.migrate(dbConfig.url, dbConfig.user, dbConfig.pass)
        } yield r
        test.map(result => assert(result.migrationsExecuted === 0))
    }

    test("DatabaseMigrator must throw an exception if the database is not available".tag(NeedsDatabase)) {
        val migrator = new DatabaseMigrator[IO]
        val test     = migrator.migrate("jdbc:nodriver://", "", "")
        test.attempt.map(r => assert(r.isLeft))
    }
}
