/*
 * Copyright (C) 2022  Contributors as noted in the AUTHORS.md file
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.smederee.tickets.config

import com.comcast.ip4s.*
import com.typesafe.config.*
import org.http4s.Uri
import org.http4s.implicits.*
import pureconfig.*

import munit.*

final class SmedereeTicketsConfigurationTest extends FunSuite {
    val rawDefaultConfig = new Fixture[Config]("defaultConfig") {
        def apply() = ConfigFactory.load(getClass.getClassLoader)
    }

    override def munitFixtures = List(rawDefaultConfig)

    test("must load from the default configuration successfully") {
        ConfigSource
            .fromConfig(rawDefaultConfig())
            .at(s"${SmedereeTicketsConfiguration.location.toString}")
            .load[SmedereeTicketsConfiguration] match {
                case Left(errors) => fail(errors.toList.mkString(", "))
                case Right(_)     => assert(true)
            }
    }

    test("default values for external linking must be setup for local development") {
        ConfigSource
            .fromConfig(rawDefaultConfig())
            .at(s"${SmedereeTicketsConfiguration.location.toString}")
            .load[SmedereeTicketsConfiguration] match {
                case Left(errors) => fail(errors.toList.mkString(", "))
                case Right(cfg) =>
                    val externalCfg = cfg.external
                    assertEquals(externalCfg.host, host"localhost")
                    assertEquals(externalCfg.port, Option(port"8080"))
                    assert(externalCfg.path.isEmpty)
                    assertEquals(externalCfg.scheme, Uri.Scheme.http)
            }
    }

    test("default values for hub service integration must be setup for local development") {
        ConfigSource
            .fromConfig(rawDefaultConfig())
            .at(s"${SmedereeTicketsConfiguration.location.toString}")
            .load[SmedereeTicketsConfiguration] match {
                case Left(errors) => fail(errors.toList.mkString(", "))
                case Right(cfg) =>
                    val expectedUri = uri"http://localhost:8080"
                    assertEquals(cfg.hub.baseUri, expectedUri)
            }
    }
}
